﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using Buzz.MachineInterface;
using BuzzGUI.Interfaces;
using BuzzGUI.Common;
using BuzzGUI.Common.InterfaceExtensions;

namespace patternist
{
    public class Assignment : IComparable<Assignment>, INotifyPropertyChanged
    {
        PatternistMachine patternistMachine;

        int sourceNote;
        public int SourceNote
        {
            get { return sourceNote; }
            set { sourceNote = value; PropertyChanged.Raise(this, "SourceNoteName"); }
        }

        ISequence targetSeq;
        public ISequence TargetSeq
        {
            get { return targetSeq; }
            set { targetSeq = value; PropertyChanged.Raise(this, "TargetSeqNum"); }
        }

        IPattern targetPat;
        public IPattern TargetPat
        {
            get { return targetPat; }
            set 
            {
                if (targetPat != null)
                {
                    targetPat.PropertyChanged -= new PropertyChangedEventHandler(pattern_PropertyChanged);
                    targetPat.Machine.PropertyChanged -= new PropertyChangedEventHandler(machine_PropertyChanged);
                }
                targetPat = value;
                targetPat.PropertyChanged += new PropertyChangedEventHandler(pattern_PropertyChanged);
                targetPat.Machine.PropertyChanged += new PropertyChangedEventHandler(machine_PropertyChanged);
                PropertyChanged.Raise(this, "TargetPatName");
                PropertyChanged.Raise(this, "TargetMacName");
            }
        }

        public string SourceNoteName { get { return BuzzNote.ToString(SourceNote); } }
        public string TargetMacName { get { return TargetPat.Machine.Name; } }
        public string TargetPatName { get { return TargetPat.Name; } }

        public int TargetSeqNum
        {
            get
            {
                if (TargetSeq == null)
                    return 0;
                var mac = TargetPat.Machine;
                var sequences = Global.Buzz.Song.Sequences.Where(s => s.Machine == mac).ToList();
                var seqNum = sequences.IndexOf(TargetSeq);
                return seqNum + 1; // 0 if not found
            }
        }


        public enum StatusType { playing, planned, off }; // plan to stop, planned while playing, countdown...?
        StatusType status = StatusType.off;
        public string StatusString
        {
            get
            {
                switch (status)
                {
                    case StatusType.off: return "";
                    case StatusType.planned: return "⌛";
                    case StatusType.playing: return "▷";
                    default: return "err";
                }
            }
        }

        int triggerTime;
        public void setTriggerTime(int t)
        {
            triggerTime = t;
            Global.Buzz.Song.PropertyChanged += new PropertyChangedEventHandler(Song_PropertyChanged);
            // * -> planned
            status = StatusType.planned;
            PropertyChanged.Raise(this, "StatusString");
        }

        void Song_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "PlayPosition" && Global.Buzz.Song.PlayPosition == triggerTime)
            {
                Global.Buzz.DCWriteLine("reached triggertime");
                Global.Buzz.Song.PropertyChanged -= new PropertyChangedEventHandler(Song_PropertyChanged);
                // planned -> playing
                status = StatusType.playing;
                PropertyChanged.Raise(this, "StatusString");
            }
        }

        public void updateStatus()
        {
            if (status == StatusType.playing && targetSeq.PlayingPattern != targetPat)
            {
                // playing -> off (pattern end or song stop)
                status = StatusType.off;
                PropertyChanged.Raise(this, "StatusString");
            }
            if (status == StatusType.planned && !Global.Buzz.Playing)
            {
                // planned -> off (song stop)
                status = StatusType.off;
                PropertyChanged.Raise(this, "StatusString");
            }
            if (status == StatusType.off && targetSeq.PlayingPattern == targetPat)
            {
                // off -> playing (external trigger)
                status = StatusType.playing;
                PropertyChanged.Raise(this, "StatusString");
            }
        }


        public Assignment(PatternistMachine _patternistMachine)
        {
            patternistMachine = _patternistMachine;
        }

        public Assignment(PatternistMachine _patternistMachine, Saveable load)
        {
            patternistMachine = _patternistMachine;
            SourceNote = load.sourceNote;
            var mac = Global.Buzz.Song.Machines.FirstOrDefault(m => m.Name == load.targetMacName);
            TargetPat = mac.Patterns.FirstOrDefault(p => p.Name == load.targetPatName); ;
            TargetSeq = Global.Buzz.Song.Sequences.Where(s => s.Machine == mac).ToList()[load.targetSeqNum - 1];
        }

        void machine_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Name")
                PropertyChanged.Raise(this, "TargetMacName");
        }

        void pattern_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Name")
                PropertyChanged.Raise(this, "TargetPatName");
        }

        public void updateSequence()
        {
            var mac = TargetPat.Machine;
            var sequences = Global.Buzz.Song.Sequences.Where(s => s.Machine == mac).ToList();
            if (sequences.IndexOf(TargetSeq) == -1)
                patternistMachine.assignments.Remove(this);
            PropertyChanged.Raise(this, "TargetSeqNum");
        }

        public struct Saveable
        {
            public int sourceNote;
            public string targetMacName;
            public string targetPatName;
            public int targetSeqNum;
        }

        public Saveable toSaveable()
        {
            return new Saveable
            {
                sourceNote = SourceNote,
                targetMacName = TargetMacName,
                targetPatName = TargetPatName,
                targetSeqNum = TargetSeqNum
            };
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public int CompareTo(Assignment other)
        {
            if (other == null)
                return 1;
            else
                return this.SourceNote.CompareTo(other.SourceNote);
        }
    }
}
