﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using BuzzGUI.Interfaces;
using BuzzGUI.Common;
using BuzzGUI.Common.InterfaceExtensions;

namespace patternist
{
    /// <summary>
    /// Interaction logic for PatternistGUI.xaml
    /// </summary>
    public partial class PatternistGUI : System.Windows.Controls.UserControl, IMachineGUI, INotifyPropertyChanged
    {
        IMachine machine;
        PatternistMachine patternistMachine;
        IParameter barParameter;
        IParameter loopParameter;
        IParameter breakParameter;

        public int BarBinding
        {
            get { return barParameter.GetValue(0); }
            set
            {
                // work around possible bug in NumericUpDown
                barParameter.SetValue(0, (int)Math.Max(BarNumeric.Minimum, Math.Min(BarNumeric.Maximum, value)));
                PropertyChanged.Raise(this, "BarBinding");
            }
        }

        public bool LoopBinding
        {
            get { return loopParameter.GetValue(0)==1; }
            set { loopParameter.SetValue(0, value ? 1 : 0); PropertyChanged.Raise(this, "LoopBinding"); }
        }

        public int BreakBinding
        {
            get { return breakParameter.GetValue(0); }
            set { breakParameter.SetValue(0, value); PropertyChanged.Raise(this, "BreakBinding"); }
        }

        public IMachine Machine
        {
            get { return machine; }
            set
            {
                if (machine != null)
                {
                    // release old machine (machine deleted)
                    barParameter.UnsubscribeEvents(0, BarChanged, null);
                    this.DataContext = null;
                    patternistMachine.Gui = null;
                    patternistMachine = null;
                }

                machine = value;

                if (machine != null)
                {
                    // bind with new machine
                    patternistMachine = (PatternistMachine)machine.ManagedMachine;
                    this.DataContext = this;
                    dataGrid.ItemsSource = patternistMachine.assignments;
                    patternistMachine.Gui = this;

                    barParameter = machine.GetParameter("Bar");
                    loopParameter = machine.GetParameter("Loop");
                    breakParameter = machine.GetParameter("Break");
                    barParameter.SubscribeEvents(0, BarChanged, null);
                    loopParameter.SubscribeEvents(0, LoopChanged, null);
                    breakParameter.SubscribeEvents(0, BreakChanged, null);
                }
            }
        }

        public PatternistGUI()
        {
            InitializeComponent();
            dataGrid.SelectedCellsChanged += new SelectedCellsChangedEventHandler(dataGrid_SelectedCellsChanged);
        }

        void dataGrid_SelectedCellsChanged(object sender, SelectedCellsChangedEventArgs e)
        {
            if (dataGrid.SelectedCells.Count == 0)
            {
                editButton.IsEnabled = false;
                removeAssignmentButton.IsEnabled = false;
            }
            else
            {
                editButton.IsEnabled = true;
                removeAssignmentButton.IsEnabled = true;
            }
        }

        private void addAssignmentButton_Click(object sender, RoutedEventArgs e)
        {
            AssignmentDialog dialogBox = new AssignmentDialog(null);
            dialogBox.Width = 410;
            dialogBox.Height = 332;
            Nullable<bool> dialogResult = dialogBox.ShowDialog();
            if (dialogResult == true)
            {
                var newAss = new Assignment(patternistMachine);
                newAss.TargetPat = dialogBox.SelectedPattern;
                newAss.TargetSeq = dialogBox.SelectedSequence;
                newAss.SourceNote = dialogBox.SelectedNote;
                patternistMachine.assignments.Add(newAss);
                patternistMachine.sortAssignments();
            }
        }

        private void editButton_Click(object sender, RoutedEventArgs e)
        {
            if (dataGrid.SelectedCells.Count==0)
                return;
            var ass = (Assignment)dataGrid.SelectedCells[0].Item;
            AssignmentDialog dialogBox = new AssignmentDialog(ass);
            dialogBox.Width = 410;
            dialogBox.Height = 332;
            Nullable<bool> dialogResult = dialogBox.ShowDialog();
            if (dialogResult == true)
            {
                ass.TargetPat = dialogBox.SelectedPattern;
                ass.TargetSeq = dialogBox.SelectedSequence;
                ass.SourceNote = dialogBox.SelectedNote;
                patternistMachine.sortAssignments();
            }
        }

        private void removeAssignmentButton_Click(object sender, RoutedEventArgs e)
        {
            if (dataGrid.SelectedCells.Count==0)
                return;
            var ass = (Assignment)dataGrid.SelectedCells[0].Item;
            patternistMachine.assignments.Remove(ass);
        }

        void BarChanged(IParameter p, int track) { PropertyChanged.Raise(this, "BarBinding"); }
        void LoopChanged(IParameter p, int track) { PropertyChanged.Raise(this, "LoopBinding"); }
        void BreakChanged(IParameter p, int track) { PropertyChanged.Raise(this, "BreakBinding"); }

        public event PropertyChangedEventHandler PropertyChanged;

        private void dataGrid_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            var ass = (Assignment)dataGrid.SelectedCells[0].Item;
            switch (dataGrid.CurrentCell.Column.DisplayIndex)
            {
            case 0:
                editButton_Click(null, null);
                return;
            case 1:
                var mac = Global.Buzz.Song.Machines.FirstOrDefault(m => m.Name == ass.TargetMacName);
                mac.DoubleClick();
                return;
            case 2:
                Global.Buzz.ActiveView = BuzzView.PatternView;
                Global.Buzz.SetPatternEditorPattern(ass.TargetPat);
                return;

            }
        }
    }

    [MachineGUIFactoryDecl(PreferWindowedGUI = true, IsGUIResizable = true)]
    public class MachineGUIFactory : IMachineGUIFactory
    {
        public IMachineGUI CreateGUI(IMachineGUIHost host) { return new PatternistGUI(); }
    }
}
