﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Windows.Data;
using System.Windows.Input;
using Buzz.MachineInterface;
using BuzzGUI.Interfaces;
using BuzzGUI.Common;

namespace patternist
{
    [MachineDecl(Name = "pac patternist", ShortName = "patternist", Author = "Pierre Cusa", MaxTracks = 8)]
    public class PatternistMachine : IBuzzMachine
    {
        public IBuzzMachineHost host;
        public PatternistGUI Gui = null;
        public IEnumerable<IMenuItem> Commands { get; private set; }
        MenuItemVM guiMenuItem;
        MenuItemVM aboutMenuItem;
		Track[] tracks = new Track[32];
        public BindingList<Assignment> assignments = new BindingList<Assignment>();
        BuzzListener buzzListener;

        [ParameterDecl(MinValue = 1, MaxValue = 128, DefValue = 16)]
        public int Bar { get; set; }

        [ParameterDecl(DefValue = false)]
        public bool Loop { get; set; }

        [ParameterDecl(MinValue = 0, MaxValue = 2, DefValue = 0)]
        public int Break { get; set; }
        public enum BreakSetting { never, onRelease, onRepeat };

        [ParameterDecl(IsStateless = true)]
        public void Note(Note v, int track) { tracks[track].Note(v); }


        public PatternistMachine(IBuzzMachineHost _host)
		{
			host = _host;
            buzzListener = new BuzzListener(this);
			
			for (int i = 0; i < tracks.Length; i++)
                tracks[i] = new Track(this);

            guiMenuItem = new MenuItemVM
            {
                ID = 0,
                Text = "Show GUI",
                Command = new DelegateCommand(() => host.Machine.DoubleClick())
            };
            aboutMenuItem = new MenuItemVM
            {
                ID = 1,
                Text = "About",
                Command = new DelegateCommand(() =>
                {
                    About box = new About();
                    box.ShowDialog();
                })
            };
            Commands = new List<IMenuItem>(new IMenuItem[] { guiMenuItem, aboutMenuItem });
		}

        public void Work()
        {
            for (int t = 0; t < host.Machine.ParameterGroups[2].TrackCount; t++)
                tracks[t].Work();
            if (host.SubTickInfo.CurrentSubTick == 0 && host.SubTickInfo.PosInSubTick == 0)
                // once per tick (subtick loops even when stopped)
                assignments.Run(a => a.updateStatus());
        }

        public void Stop()
        {
            assignments.Run(a => a.TargetSeq.TriggerEvent(0, null, false));
        }

        // buzz notes also get funneled into this
        public void MidiNote(int channel, int value, int velocity)
        {
            if (!Global.Buzz.Playing)
                return;
            foreach (var assignment in assignments.Where(a => BuzzNote.ToMIDINote(a.SourceNote) == value))
            {
                var song = Global.Buzz.Song;
                var time = song.LoopStart + ((song.PlayPosition - song.LoopStart + Bar) / Bar * Bar % (song.LoopEnd - song.LoopStart));
                if (Loop && assignment.TargetSeq.PlayingPattern == assignment.TargetPat &&
                    ((Break == (int)BreakSetting.onRepeat ) && velocity != 0 ||
                     (Break == (int)BreakSetting.onRelease && velocity == 0)))
                {
                    assignment.TargetSeq.TriggerEvent(time, new SequenceEvent(SequenceEventType.Break), false);
                }
                else if (velocity != 0)
                {
                    assignment.setTriggerTime(time);
                    assignment.TargetSeq.TriggerEvent(time, new SequenceEvent(SequenceEventType.PlayPattern, assignment.TargetPat), Loop);
                }
            }
        }

        public class StateData
        {
            public List<Assignment.Saveable> assignments;
        }

        public List<Assignment.Saveable> loadingAssignments;

        public StateData MachineState
        {
            // save/export
            get
            {
                var savedAsses = assignments.Select(a => a.toSaveable()).ToList();
                return new StateData { assignments = savedAsses };
            }
            // load/import
            set
            {
                loadingAssignments = value.assignments;
            }
        }

        public void ImportFinished(IDictionary<string, string> machineNameMap)
        {
            if (loadingAssignments == null)
                return;
            assignments = new BindingList<Assignment>(loadingAssignments.Select(a => new Assignment(this, a)).ToList());
            loadingAssignments = null;
        }

        public void sortAssignments()
        {
            // where is BindingList.Sort?
            var listed = assignments.ToList();
            listed.Sort();
            assignments.Clear();
            listed.ForEach(item => assignments.Add(item));
        }

        public class DelegateCommand : ICommand
        {
            private readonly Action _execute;
            public event EventHandler CanExecuteChanged;

            public DelegateCommand(Action execute)
            { _execute = execute; }
            public bool CanExecute(object parameter)
            { return true; }
            public void Execute(object parameter)
            { _execute(); }
        }
    }
}
